@@ -10,7 +10,7 @@ The dh-golang package provides a build system for debhelper which can be used in
 the following way:
 
  %:
- 	dh $@ --buildsystem=golang --with=golang
+ 	dh $@ --builddirectory=_build --buildsystem=golang --with=golang
 
 =head1 IMPLEMENTATION
 
@@ -281,7 +281,8 @@ sub _set_dh_gopkg {
     #                             gopkg.in/mgo.v2,
     #                             labix.org/v2/mgo,
     #                             launchpad.net/mgo
-    $ENV{DH_GOPKG} = (split ",", $source->{"XS-Go-Import-Path"})[0];
+    my $import = $source->{"XS-Go-Import-Path"} =~ s/\n/ /gr;
+    $ENV{DH_GOPKG} = (split ",", $import)[0];
 }
 
 sub _set_gopath {
@@ -313,6 +314,58 @@ sub _set_goproxy {
     $ENV{GOPROXY} = "off";
 }
 
+my %GOOS_MAPPING = (
+    'linux' => 'linux',
+);
+my %GOARCH_MAPPING = (
+    'amd64/base'     => {'GOARCH' => 'amd64'},
+    'amd64/x32'      => {'GOARCH' => 'amd64p32'},
+    'arm/eabi'       => {'GOARCH' => 'arm', 'GOARM' => '5'},
+    'arm/eabihf'     => {'GOARCH' => 'arm', 'GOARM' => '7'},
+    'arm64/base'     => {'GOARCH' => 'arm64'},
+    'i386/base'      => {'GOARCH' => '386'},
+    'mips/base'      => {'GOARCH' => 'mips'},
+    'mips64/abi64'   => {'GOARCH' => 'mips64'},
+    'mips64el/abi64' => {'GOARCH' => 'mips64le'},
+    'mipsel/base'    => {'GOARCH' => 'mipsle'},
+    'powerpc/base'   => {'GOARCH' => 'ppc'},
+    'ppc64/base'     => {'GOARCH' => 'ppc64'},
+    'ppc64el/base'   => {'GOARCH' => 'ppc64le'},
+    'riscv64/base'   => {'GOARCH' => 'riscv64'},
+    's390/base'      => {'GOARCH' => 's390'},
+    's390x/base'     => {'GOARCH' => 's390x'},
+    'sparc/base'     => {'GOARCH' => 'sparc'},
+    'sparc64/base'   => {'GOARCH' => 'sparc64'},
+);
+
+sub _set_gocross {
+    return unless is_cross_compiling();
+
+    unless ($ENV{CC}) {
+        $ENV{CC} = dpkg_architecture_value("DEB_HOST_GNU_TYPE") . "-gcc";
+    }
+    unless ($ENV{PKG_CONFIG}) {
+        $ENV{PKG_CONFIG} = dpkg_architecture_value("DEB_HOST_GNU_TYPE") . "-pkg-config";
+    }
+
+    $ENV{CGO_ENABLED} = "1";
+
+    my $host_os = dpkg_architecture_value("DEB_HOST_ARCH_OS");
+    if (defined(my $goos = $GOOS_MAPPING{$host_os})) {
+        $ENV{GOOS} = $goos;
+    } else {
+        error("Cannot cross-compile: Missing entry for HOST OS ${host_os}.");
+    }
+
+    my $host_cpu_abi = dpkg_architecture_value("DEB_HOST_ARCH_CPU") . "/" .
+                       dpkg_architecture_value("DEB_HOST_ARCH_ABI");
+    if (defined(my $cpu_vars = $GOARCH_MAPPING{$host_cpu_abi})) {
+        @ENV{keys %{$cpu_vars}} = @{$cpu_vars}{keys %{$cpu_vars}};
+    } else {
+        error("Cannot cross-compile: Missing entry for CPU/ABI ${host_cpu_abi}.");
+    }
+}
+
 sub _link_contents {
     my ($src, $dst) = @_;
 
@@ -480,13 +533,24 @@ sub build {
     $this->_set_gocache();
     $this->_set_go111module();
     $this->_set_goproxy();
+    $this->_set_gocross();
     if (exists($ENV{DH_GOLANG_GO_GENERATE}) && $ENV{DH_GOLANG_GO_GENERATE} == 1) {
         $this->doit_in_builddir("go", "generate", "-v", @_, get_targets());
     }
     unshift @_, ('-p', $this->get_parallel());
-    # Go 1.10 changed flag behaviour, -{gc,asm}flags=all= only works for Go >= 1.10.
-    my $trimpath = "all=\"-trimpath=" . $ENV{GOPATH} . "/src\"";
-    $this->doit_in_builddir("go", "install", "-gcflags=$trimpath", "-asmflags=$trimpath", "-v", @_, get_targets());
+
+    my ($minor) = (qx(go version) =~ /go version go1\.([0-9]+)/);
+    if ($minor >= 13) {
+        # Go 1.13 officially supports reproducible build, adding new -trimpath option
+        # https://github.com/golang/go/issues/16860
+        $this->doit_in_builddir("go", "install", "-trimpath", "-v", @_, get_targets());
+    } elsif ($minor >= 10) {
+        # Go 1.10 changed flag behaviour, -{gc,asm}flags=all= only works for Go >= 1.10.
+        my $trimpath = "all=\"-trimpath=" . $ENV{GOPATH} . "/src\"";
+        $this->doit_in_builddir("go", "install", "-gcflags=$trimpath", "-asmflags=$trimpath", "-v", @_, get_targets());
+    } else {
+        $this->doit_in_builddir("go", "install", "-v", @_, get_targets());
+    }
 }
 
 sub test {
@@ -532,7 +596,12 @@ sub install {
     my @binaries = glob "$builddir/bin/*";
     if ($install_binaries and @binaries > 0) {
         $this->doit_in_builddir('mkdir', '-p', "$destdir/usr");
-        $this->doit_in_builddir('cp', '-r', 'bin', "$destdir/usr");
+        if (is_cross_compiling()) {
+            $this->_set_gocross();
+            $this->doit_in_builddir('cp', '-r', '-T', "bin/$ENV{GOOS}_$ENV{GOARCH}", "$destdir/usr/bin");
+        } else {
+            $this->doit_in_builddir('cp', '-r', 'bin', "$destdir/usr");
+        }
     }
 
     if ($install_source) {
